#! /usr/bin/python3

import argparse

import sys
import os
import csv
#import ipdb
import testeur
import pandas
import json
import re
import uuid
import time
import numpy as np

# Configuration
default_host = "localhost"
default_user = "ubuntu"

dockers = {}

nbOK = 0
nbNOK = 0

def ok(msg):
    print ("\033[48;5;100m\033[1m{}\033[0m\t{}".format("OK", msg))

def nok(msg):
    print ("\033[48;5;196m\033[1m\33[5m{}\033[0m\t{}".format("Not OK", msg))


def get_descript(t, iddocker):
    """
    Récupère une descrition d'un docker dont on a le nom a priori a partir
    de la commande docker inspect
    """
    sortie, err = t.test("sudo docker inspect {}".format(iddocker))
    if len(err) != 0:
        raise ValueError("Le docker {} n'existe pas".format(iddocker))
    descr = {}
    inspect = json.loads("".join(sortie))[0]

    descr['Name'] = inspect['Name']
    descr['HostnamePath'] = inspect['HostnamePath']
    descr['HostsPath'] = inspect['HostsPath']

    descr['Volumes'] = inspect['Mounts']
    descr['Ports'] = inspect['NetworkSettings']['Ports']
    descr['Networks'] = {}
    for net in inspect['NetworkSettings']['Networks'].keys():
        descr['Networks'][net] = {}
        descr['Networks'][net]['IPAddress'] = inspect['NetworkSettings']['Networks'][net]['IPAddress']
        descr['Networks'][net]['Prefix'] = inspect['NetworkSettings']['Networks'][net]['IPPrefixLen']
        descr['Networks'][net]['Aliases'] = inspect['NetworkSettings']['Networks'][net]['Aliases']

    descr['Env'] = inspect['Config']['Env']
    descr['ExtraHosts'] = inspect['HostConfig']['ExtraHosts']
    return descr

def change_status_docker(nom, status, t):
    r,e = t.test("docker {} {}".format(status, nom))
    if (len(e) != 0):
        print("La commande docker {} {} a occasionner l'erreur {}".format(status, nom, '\n'.join(e)))


def existance_fichiers(t, fichiers):
    manque = []
    for fich in fichiers:
        if not os.path.exists(fich):
            manque.append(fich)
    return manque




def test_presence_services(t, dat):
    """
    teste si les services décrit par dat existent
    
    retourne pour chaque servic trouvé, le nom des dockers et leur
    machines de support.
    """
    sortie, err = t.test("sudo docker service ls", result="file")
    sort = pandas.read_fwf(sortie,header=0)

    
    print("Services trouvés :\n");
    for l in sort['NAME']:
        print("\t\"{}\"".format(l))

    services = {}
    nbtok = 0
    nbtnok = 0
    neuds_utilises = []
    for key in dat.keys():
        # le docker est-il dans la liste des noms
        regnom = dat[key]['NAME']
        trouv = sort[sort['NAME'].map(lambda x: bool(re.match(regnom,x)))]
        nbtest = 2
        if len(trouv) == 1:
            pos = trouv.index[0]
            nom = str(trouv.loc[pos,'NAME'])
            ok("Service \"{}\" trouvé en position {}".format(nom, pos))
            nbtok=nbtok+1

            bon = True
            regrep = dat[key]['REPLICAS']
            if re.match(regrep, trouv.loc[pos, 'REPLICAS']):
                ok("Le service {} a bien {} réplicas".format(nom, regrep))
            else:
                nok("Le service {} a {} réplicas".format(nom,  trouv.loc[pos, 'REPLICAS']))
                bon = False
            regimage = dat[key]['IMAGE']
            if re.match(regimage, trouv.loc[pos, 'IMAGE']):
                ok("Le service {} a bien l'image {} ".format(nom, trouv.loc[pos, 'IMAGE']))
            else:
                nok("Le service {} a l'image {} qui ne convient pas à {}"
                    .format(nom,  trouv.loc[pos, 'IMAGE'], regimage))
                bon = False

            if (bon):
                nbtok=nbtok+1
            else:
                nbtnok=nbtnok+1

            service = {}
            service['nom'] = nom
            service['image'] = trouv.loc[pos, 'IMAGE']
            service['ports'] = trouv.loc[pos, 'PORTS']
            sortie, err = t.test("sudo docker service ps --format '\"{{{{.Name}}}}\",\"{{{{.Node}}}}\",\"{{{{.CurrentState}}}}\"' {}".format(nom),
                                 result='file')
            tab_dockers = pandas.read_csv(sortie,header=None,sep=',', quotechar='"')
            dockers = []
            for index, row in tab_dockers.iterrows():
                if (re.match('Running .*', row[2])):
                    dockers.append({'name':row[0], 'node':row[1]})
                    neuds_utilises.append(row[1])
            service['dockers'] = dockers
            services[key] = service
        else:
            nok("Le service {} a été trouvé {} fois via l'expression régulière {}"
                .format(key, len(trouv), regnom))
            nbtnok = nbtnok+2

    nutil = np.unique(neuds_utilises)
    if (len(nutil)) == 3:
        ok("Il y a bien 3 neuds utilisés dans le clusters (3pts)")
        nbtok = nbtok+3
    else:
        nok("Il n'y a pas 3 neouds utilisées dans le clusters les seuls sont {} (-3points)".format(nutil))
        nbtnok = nbtnok+3
    
    print("-----------------------------------------")
    print("Résultats du test : {} bon sur {}".format(nbtok, nbtok+nbtnok))
    print("-----------------------------------------")
    return (nbtok, nbtnok, services)

def get_descr_nodes(t):
    sortie, err = t.test("sudo NO_PROXY=.univ-lyon1.fr docker node ls --format '\"{{.Hostname}}\",\"{{.ManagerStatus}}\",\"{{.Status}}\"'", result="file")
    sort = pandas.read_csv(sortie,header=None,sep=',',quotechar='"')

    nodes = {}
    for index, row in sort.iterrows():
        nodes[row[0]] = {'nom':row[0], 'leader':(row[1]=="Leader"), 'status':row[2]}

    return nodes

def test_docker_machine(t, services, dat):
    """
    teste si les machines décrites par dat existent
    """
    sortie, err = t.test("sudo NO_PROXY=.univ-lyon1.fr docker-machine ls", result="file")
    sort = pandas.read_fwf(sortie,header=0)
    
    print("Machines trouvées :\n");
    for l in sort['NAME']:
        print("\t\"{}\"".format(l))

    nodes = get_descr_nodes(t)
    
    machines = {}
    nbtok = 0
    nbtnok = 0

    if (len(nodes.keys()) == 3):
        ok("Il y a bien 3 neuds dans le clusters")
        nbtok = nbtok+1
    else:
        nok("Il y a {} noeuds dans le cluster".format(len(nodes.keys())))
        nbtnok = nbtnok+1
    

    neuds_utilises = []
    for key in dat.keys():
        # le docker est-il dans la liste des noms
        regnom = dat[key]['NAME']
        trouv = sort[sort['NAME'].map(lambda x: bool(re.match(regnom,x)))]
        if len(trouv) == 1:
            pos = trouv.index[0]
            nom = str(trouv.loc[pos,'NAME'])
            ok("Machine \"{}\" trouvée en position {}".format(nom, pos))
            nbtok=nbtok+1

            bon = True
            m = re.match("tcp://(.*):2376", trouv.loc[pos, 'URL'])
            if (m):
                ip = m.group(1)
                ok("J'ai trouvé l'IP : {}".format(ip))
            else:
                nok("Je n'arrive pas à trouver son IP dans l'URL {}".format( trouv.loc[pos, 'URL']))


            if (nom in nodes.keys()):
                ok("La machine fait partie du cluster")
                if (nodes[nom]['status'] == 'Ready'):
                    ok("Elle est bien active")
                else:
                    nok("Son status est {} au lieu de Ready".format(nodes[nom]['status']))
                    bon = False
                if (not nodes[nom]['leader']):
                    ok("Elle n'est pas manager")
                else:
                    nok("Son status est manager au lieu de worker")
                    bon = False
            else:
                nok("La machine ne fait pas partie du cluster qui contient {}".format(node.keys()))
                bon = False
                
            if bon:
                nbtok = nbtok+2
            else:
                nbtnok = nbtnok+2
            
            
            machine = {}
            machine['nom'] = nom
            machine['ip'] = ip
            machines[key] = machine
        else:
            nok("Le machine {} a été trouvé {} fois via l'expression régulière {}"
                .format(key, len(trouv), regnom))
            nbtnok = nbtnok+2
    
    print("-----------------------------------------")
    print("Résultats du test : {} bon sur {}".format(nbtok, nbtok+nbtnok))
    print("-----------------------------------------")
    return (nbtok, nbtnok, machines)
    



    
    
def test_all(server="localhost", user=None, key=None, parts = ["debut", "owncloud"]):
    totalOK=0
    totalNOK=0
    
    if server == "localhost":
        t = testeur.testeur()
    else:
        t = testeur.testeur_ssh(server, user, key=key)

    services_a_tester = {
        "regbrowser" : {
            "NAME" : "regbrowser",
            "REPLICAS" : "1/1",
            "IMAGE" : "konradkleine/docker-registry-frontend.*"
        },
        "registry" : {
            "NAME" : "registry",
            "REPLICAS" : "1/1",
            "IMAGE" : "registry.*"
        },
        "stackphp" : {
            "NAME" : "stackphp.*",
            "REPLICAS" : "2/2",
            "IMAGE" : "localhost:5000/.*"
        },
        "viz" : {
            "NAME" : "viz",
            "REPLICAS" : "1/1",
            "IMAGE" : "dockersamples/visualizer.*"
        }
    }

    (nbok, nbnok, services) = test_presence_services(t, services_a_tester)
    totalOK = totalOK+nbok
    totalNOK = totalNOK + nbok

    machines_a_verifier = {
        'worker1' : {
            'NAME':'(.*-work1|worker.*1)'
        },
        'worker2' : {
            'NAME':'(.*-work2|worker.*2)'
        }
    }
    (nbok, nbnok, services) = test_docker_machine(t, services, machines_a_verifier)
    totalOK = totalOK+nbok
    totalNOK = totalNOK + nbok
    
    t.close()
    return (nbOK,nbNOK)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Passe des scripts de test à distance ou en local")
    parser.add_argument("-s", "--server", type=str, dest="server", default=default_host, help="La machine sur laquelle se connecter")
    parser.add_argument("-u", "--user", type=str, dest="user",
                        default=default_user, help="Le nom de l'utilisateur pour se connecter sur la machine (n'est pas considéré pour la machine locale")
    parser.add_argument("-k", "--key", type=str, dest="key", default=None, help="nom du fichier de clef")

    parser.add_argument("parts", type=str, metavar="part", default=["all"], nargs='*', 
                        help="Parties de la vérification à faire all")
    
    args = parser.parse_args()

    print("##################################")
    print("##################################")
    print("Test du serveur {} pour {}".format(args.server, " ".join(args.parts)))
    print("##################################")
    print("##################################")
    
    nbok, nbnok = test_all(args.server, args.user,args.key, args.parts)
    print ("## Au final {}/{}".format(nbOK, nbOK+nbNOK))
    
